import QtQuick 1.0
import QtWebKit 1.0
import "Util.js" as Util

Rectangle {

    id: advertContainer

    property string imageUrl
    property string clickUrl
    property string advertUrl: getTrackUrl()

    signal advertClicked(string advertClickUrl)

    Behavior on height { NumberAnimation { duration: 200  } }

    states: [
          State {
              name: "reloadAdvert"
              when: appState.loading == true

              StateChangeScript { script: reload() }

          }
      ]


    onAdvertUrlChanged: {

        //Util.log("on advert url changed")
        if ((advertUrl != null) && (advertUrl.length > 0)) {

            getAdvertHtml()
        }
    }

    function reload(){
           if ((advertUrl != null) && (advertUrl.length > 0)) {

               getAdvertHtml()
           }
       }


    function getAdvertHtml() {

        if ((advertUrl != null) && (advertUrl.length > 0)) {

            //dont try if we are offline
            if (!props.theme.iAmOnline) {
                Util.log("*************************************** OFFLINE - DONT DOWNLOAD AD");
                return;
            }

            // download rss xml,
            var xh = new XMLHttpRequest()

            //Util.log("advert url is " + advertUrl)

            xh.open("GET", advertUrl, true)

            xh.onreadystatechange = function() {

                if (xh.readyState == XMLHttpRequest.DONE) {

                    //Util.log("response is " + xh.responseText)

                    if(xh.responseXML != null){

                        var root = xh.responseXML.documentElement;
                        var channel = root.childNodes[1];
                        for (var i = 0; i < channel.childNodes.length; i++ ) {
                            var baby = channel.childNodes[i]
                            if (baby.nodeName == "item") {
                                var advertNode = baby.childNodes[1];

                                // check if advertNode child is defined
                                if ((advertNode.nodeName == "advert-html") &&
                                        (advertNode.childNodes[0])) {

                                    var advertHtml = advertNode.childNodes[0].nodeValue;
                                    //Util.log("advert html is " + advertHtml);

                                    // get click URL
                                    var search1 = advertHtml.indexOf("openURL('") + 9;
                                    var search2 = advertHtml.indexOf("')");

                                    //strange behaviour!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
                                    clickUrl = advertHtml.substring(search1,search2);


                                    // it is all CDATA so cannot use XML parsing
                                    var advertHtmlSections = advertHtml.split("\"");
                                    for (var j = 0; j < advertHtmlSections.length; j++) {

                                        //Util.log("what it is " + advertHtmlSections[j])
                                        if(" width=" == advertHtmlSections[j]){
                                            j++;
                                            advertContainer.width = advertHtmlSections[j];
                                            //Util.log("width is " + advertContainer.width)

                                        } else if (" height=" == advertHtmlSections[j]) {

                                            j++;
                                            advertContainer.height = advertHtmlSections[j];
                                            //Util.log("height is " + advertContainer.height)

                                        } else if ("><img src=" == advertHtmlSections[j]) {

                                            j++;
                                            advertContainer.imageUrl = advertHtmlSections[j].replace(/&amp;/g, "&");
                                            //Util.log("decoded url is " + advertContainer.imageUrl)
                                        }
                                    }
                                }
                            }
                        }
                    } else {

                        Util.log("response doc is not valid:: " + xh.responseText);
                    }
                }
            }
            xh.send();
        }
    }

    function getTrackUrl(){
        return props.theme.advertPrefix
    }


    // child objects
    Image {
        id: advertImage
        anchors.fill: advertContainer
        anchors.topMargin: 5
        source: imageUrl
        onStatusChanged: displayAdvert()

    }

    function displayAdvert(){
        if(advertImage.status == Image.Error  || advertImage.status == Image.Null){
            advertContainer.height = 0
        }
    }

    MouseArea {

        id:webViewTouch

        anchors.fill: advertContainer
        onClicked:{
            //Util.log("advert on click fired")
            advertClicked(clickUrl)
        }
    }
}



