import QtQuick 1.1
import "Util.js" as Util
import com.meego 1.0


Rectangle {
    id: container

    property string fontName: "Helvetica"
    property string itemTitle
    property string itemDescription
    property string itemUrl
    property string itemLink
    property string itemImageUrl
    property string itemPubmsago
    property string itemMippinLink
    property bool captionIsActive: false
    property int heightOfTitle
    property int footerHeight:  72
    property string itemVideo:""

    signal itemSelected(int index)
    signal loadWebView(string urlRes, bool isOrigStory)

    onItemLinkChanged: {

        //Util.log("item link is " + itemLink + " >> " + (itemLink != "null"))
        linkButton.visible = (itemLink != "null")
    }

    width: mainWindow.width
    height: mainWindow.height

    color: "black"

    BusyRemoteImage {

        id: imagePart

        z:150
        imageUrl:itemImageUrl
        width: 450
        anchors.fill: parent
        fillMode: Image.PreserveAspectFit
        smooth: true

        MouseArea{

            id:seeVideo

            width: 200
            height: 200
            z:300
            anchors.centerIn: imagePart
            visible: itemVideo != undefined && itemVideo != ""? true:false
            onClicked:{
                if(itemVideo != undefined && itemVideo != ""){
                    container.loadWebView(itemVideo,true)
                    //showText()
                }
            }
        }
    }

    Image{
        id:brokenImage
        z:150
        anchors.centerIn: parent
        visible: imagePart.status == Image.Error || container.itemImageUrl == ""
        source: props.theme.images.brokenImage
    }




    MouseArea{
        id:mouseArea
        anchors.fill: flick

        onClicked: showText()

    }

    function showText(){
        if(captionIsActive){
            captionIsActive = false
            appState.titleBarIsVisible = false
            appState.footerIsVisible = false
            fadeOutAnimation.start()
        }
        else{
            //Util.log("IS ARTICLE GALLERY??? : " + appState.isArticleGallery)
            captionIsActive = true
            appState.titleBarIsVisible = false
            appState.footerIsVisible = true
            fadeInAnimation.start()
        }
    }

    Item{
        //color:"transparent"
        id:backgroundLayer

        z:175
        height: mainWindow.height
        width: mainWindow.width
        visible: captionIsActive

        Image {
            anchors.fill: parent
            source:mainWindow.height > mainWindow.width ? props.theme.images.dimmer_gallery_meego : props.theme.images.dimmer_gallery_meego_ls
        }
    }



    Flickable{
        id:flick
        visible: true
        opacity: 0
        z:200
        flickableDirection: "VerticalFlick"
        clip: true
        contentHeight:  titleText.height + titleText.anchors.topMargin + timestamp.height + timestamp.anchors.topMargin + textPart.height + textPart.anchors.topMargin + linkButton.height + footerHeight
        interactive:true
        anchors.fill: parent


        Text {
            id: titleText
            text: container.itemTitle
            anchors {
                leftMargin:8
                rightMargin:8
                topMargin:16
                top: parent.top
                left: parent.left
                right: parent.right
            }
            font {
                family: container.fontName
                pixelSize: 37
                bold: true
            }
            color: "white"
            wrapMode: Text.Wrap
            textFormat: Text.RichText
        }



        Text {
            id: timestamp
            anchors {
                top: titleText.bottom
                topMargin:16
                left: parent.left
                leftMargin:8
            }
            font {
                family: container.fontName
                pixelSize:16
            }
            color: "white"
            text: container.getTextForTimestamp(itemPubmsago)

        }




        Text {
            id: textPart
            anchors {
                top: timestamp.bottom
                topMargin:16
                left: parent.left
                right: parent.right
                rightMargin:8
                leftMargin:8
            }
            font {
                family:"Arial"
                pixelSize: 27
                //weight: Font.Light
                wordSpacing:2
            }
            color: "white"
            text: description
            wrapMode: Text.Wrap
            textFormat: Text.RichText
        }




        Rectangle{
            id:linkButton
            width: parent.width
            color: "transparent"
            z:250
            height: 60
            anchors {
                top: textPart.bottom
                left:parent.left
                right:parent.right
                leftMargin:8
            }

            Text{
                text: props.theme.seeOriginalStory
                color:props.theme.linkColor
                font {
                    pixelSize: 30
                    bold: true
                }
                anchors.fill: parent
                verticalAlignment: Text.AlignVCenter
            }
            MouseArea{
                z:270
                anchors.fill: parent
                onClicked: loadWebView(container.itemLink,true)

                }

        }



        MouseArea{
            z: 220
            id:mouseAreaFlick
            height: flick.contentHeight > container.height ? flick.contentHeight : container.height //height of the bottom bar
            width: mainWindow.width

            onClicked: {
                if(captionIsActive){
                    captionIsActive = false
                    appState.titleBarIsVisible = false
                    appState.footerIsVisible = false
                    fadeOutAnimation.start()
                }
                else{
                    captionIsActive = true
                    appState.titleBarIsVisible = false
                    appState.footerIsVisible = true
                    fadeInAnimation.start()
                }

            }
        }
    }


    property variant fadeInAnimation : PropertyAnimation { target: flick; property: "opacity"; to: 1 }
    property variant fadeOutAnimation : PropertyAnimation { target: flick; property: "opacity"; to: 0 }





    function getTextForTimestamp(aTimestamp) {

        if ((aTimestamp != null) && ("NOT SET" != aTimestamp)){

            var pubDate = new Date(aTimestamp);
            var now = new Date();
            var millisecondsAgo = now.getTime() - pubDate.getTime();
            var timeString = "";
            var minutesDiff = parseInt(millisecondsAgo / 60000);
            var hours = parseInt(minutesDiff / 60);
            var days = parseInt(hours / 24);
            var months = parseInt(days / 30);
            var years = parseInt(days / 365);
            minutesDiff = minutesDiff - (hours * 60);

            if (years > 0) {
                timeString = (years > 1 ? props.theme.yearPlural : props.theme.yearSingle).replace("{number}", years);
            } else if (months > 0) {
                timeString = (months > 1 ? props.theme.monthPlural : props.theme.monthSingle).replace("{number}", months);
            } else if (days > 0) {
                timeString = (days > 1 ? props.theme.dayPlural : props.theme.daySingle).replace("{number}", days);
            } else if (hours > 0) {
                timeString = (hours > 1 ? props.theme.hourPlural : props.theme.hourSingle).replace("{number}", hours);
            } else // do minutes
            {
                if (minutesDiff == 0)
                    timeString = props.justPublished;
                else
                    timeString = (minutesDiff > 1 ? props.theme.minutePlural : props.theme.minuteSingle).replace("{number}", minutesDiff);
            }

            return timeString;
        } else
            return '';
    }



}

