import QtQuick 1.1

import "Util.js" as Util

Flickable {
    // id
    id: container

    // declare props
    property string fontName: "Helvetica"
    property string itemTitle
    property string itemDescription
    property string itemUrl
    property string itemImageUrl:""
    property string itemPubmsago
    property string itemMippinLink
    property string itemLink
    property string itemVideo:""
    property string itemAudio1
    property string itemAudio2
    property string itemAudio3
    property variant audioArray : []
    property bool showFullDescription:false
    property int heightOfStringVideo: seeVideo.height
    property int  page : 1
    property int first_Char:0
    property bool isLastPage: true
    property int footerHeight:   72
    property int titleBarHeight:  mainWindow.height > mainWindow.width ? 120 : 60
    property int charsPerPage: 2000

    property alias seeVideo:seeVideo

    signal itemSelected(int index)
    signal loadWebView(string urlRes, bool isOrigStory)

    // JS functions
    function shouldShowDesc() {

        return ((typeof container.itemDescription != "undefined") &&
                (container.itemDescription != null) &&
                (container.itemDescription.length > 0))
    }

    function prepareDescText() {

        var FIRST_CHAR = (page - 1) * charsPerPage
        var LAST_CHAR = page * charsPerPage

        while ((itemDescription.charAt(LAST_CHAR) != " ") &&
               (LAST_CHAR < itemDescription.length)) {

            LAST_CHAR++;
        }

        if(LAST_CHAR > itemDescription.length) {

            LAST_CHAR = itemDescription.length
            isLastPage = true

        } else {

            isLastPage = false
        }

       if (shouldShowDesc()) {

            var candidateString =
                container.itemDescription.substring(FIRST_CHAR, LAST_CHAR)

            if(isLastPage == false) {

                var indexFirstSpace = candidateString.indexOf(" ")

                if(page== 1) {

                    indexFirstSpace = 0
                }
                candidateString =
                        candidateString.substring(indexFirstSpace, LAST_CHAR)
                return candidateString.concat(" ...")

            } else {

                return candidateString
            }
       }

        // send back empty string if we have nothing to display
        return ""
    }

    function getPagingIndex(){

        if (shouldShowDesc()) {

            var maxPages = Math.floor(itemDescription.length/charsPerPage) + 1
            var index = props.theme.content.replace("{number1}", page).replace("{number2}", maxPages)
            return index

        } else {

            return ""
        }
    }

    function getTextForTimestamp(aTimestamp) {

        if ((aTimestamp != null) && ("NOT SET" != aTimestamp)){

            var pubDate = new Date(aTimestamp);
            var now = new Date();
            var millisecondsAgo = now.getTime() - pubDate.getTime();
            var timeString = "";
            var minutesDiff = parseInt(millisecondsAgo / 60000);
            var hours = parseInt(minutesDiff / 60);
            var days = parseInt(hours / 24);
            var months = parseInt(days / 30);
            var years = parseInt(days / 365);
            minutesDiff = minutesDiff - (hours * 60);

            if (years > 0) {
                timeString = (years > 1 ? props.theme.yearPlural : props.theme.yearSingle).replace("{number}", years);
            } else if (months > 0) {
                timeString = (months > 1 ? props.theme.monthPlural : props.theme.monthSingle).replace("{number}", months);
            } else if (days > 0) {
                timeString = (days > 1 ? props.theme.dayPlural : props.theme.daySingle).replace("{number}", days);
            } else if (hours > 0) {
                timeString = (hours > 1 ? props.theme.hourPlural : props.theme.hourSingle).replace("{number}", hours);
            } else // do minutes
            {
                if (minutesDiff == 0)
                    timeString = props.justPublished;
                else
                    timeString = (minutesDiff > 1 ? props.theme.minutePlural : props.theme.minuteSingle).replace("{number}", minutesDiff);
            }

            return timeString;
        } else
            return '';
    }

    // Object props
    width: mainWindow.width
    height: mainWindow.height
    flickableDirection: "VerticalFlick"
    contentHeight:  audioList.height + audioList.anchors.topMargin + titleText.height + titleText.anchors.topMargin + imagePart.height + timestamp.height +
                    textPart.height  + linkButton.height +
                    + pagingButton.height + advert.height + advert.anchors.topMargin  + imagePart.anchors.topMargin +
                    + textPart.anchors.topMargin + timestamp.anchors.topMargin + linkButton.anchors.topMargin + footerHeight

    LayoutMirroring.enabled: props.theme.rightToLeftOrientation
    LayoutMirroring.childrenInherit: true

    onItemLinkChanged: {

        linkButton.visible = (itemLink != "null")
    }

    // child objects
    Advert {

        id: advert
        anchors {
            top: parent.top
            topMargin:titleBarHeight
            horizontalCenter: parent.horizontalCenter
        }
        onAdvertClicked: {
            //Util.log("article view - on advert clicked with - " + advertClickUrl)
            loadWebView(advertClickUrl,false)
        }
    }

    Text {
        id: titleText
        text: container.itemTitle
        anchors {
            leftMargin:8
            rightMargin:8
            topMargin:16
            top: advert.bottom
            left: parent.left
            right: parent.right
        }
        font {
            family: container.fontName
            pixelSize: 37
            bold: true
        }
        color: "black"
        wrapMode: Text.Wrap
        textFormat: Text.RichText
    }

    ListView{
        id:audioList
        anchors.left: parent.left
        anchors.top: imagePart.height > 0 ? imagePart.bottom : titleText.bottom
        anchors.topMargin:audioArray != undefined && audioArray.length != 0 ? 20 + heightOfStringVideo : 0
        height: audioArray != undefined ? audioArray.length * 80 : 0
        model: audioArray
        delegate:audioDelegate
    }

    Component{
        id:audioDelegate

        AudioCell{
            id:audioCell
            height: 80
            width:mainWindow.width
            label: modelData
            onLaunchAudio: loadWebView(label, true)
            textColor: props.theme.appTextColor

        }
    }

    TemplateRemoteImage {
        id: imagePart
        anchors {
            top: titleText.bottom
            left: parent.left
            leftMargin:  parent.width/2 - imagePart.width/2
            topMargin:itemImageUrl != "" ? 20 : 0
        }
        fillMode: Image.PreserveAspectFit
        imageUrl: itemImageUrl
        width: 450
        visible: itemImageUrl != ""

        Button{
            id:seeVideo
            anchors.top: imagePart.bottom
            anchors.left: imagePart.left
            width: parent.width
            height: itemVideo != undefined && itemVideo != ""? 30 : 0
            visible: itemVideo != undefined && itemVideo != "" && imagePart.height > 0 ? true:false
            onClicked:{
                if(itemVideo != undefined && itemVideo != ""){
                    container.loadWebView(itemVideo,true)
                }
            }
            Text{
                anchors.fill: parent
                text: props.theme.viewVideo
                color: props.theme.appTextColor
                horizontalAlignment: Text.AlignHCenter
                font.pixelSize: 30
                font.bold: true
            }
        }
    }

    MouseArea{
        id:mouseImage

        anchors.fill: imagePart
        onClicked:{
            if(itemVideo != undefined && itemVideo != ""){
                Util.log("THE ITEM VIDEO IS: " + itemVideo )
                container.loadWebView(itemVideo,true)
            }
        }
    }


    Text {
        id: timestamp
        anchors {
            top: audioList.height == 0 ? imagePart.bottom : audioList.bottom
            topMargin:16 + heightOfStringVideo
            left: parent.left
            leftMargin:8
        }
        font {
            family: container.fontName
            pixelSize:16
        }
        color: props.theme.timestampFontColor
        text: getTextForTimestamp(itemPubmsago)   
    }

    Text {
        id: textPart

        anchors {
            leftMargin:8
            top: timestamp.bottom
            topMargin:16
            left: parent.left
            right: parent.right
            rightMargin:8
        }
        font {
            family:"Arial"
            pixelSize: 27
            //weight: Font.Light
            wordSpacing:2
        }
        visible: true
        color: "black"
        text: container.prepareDescText()
        wrapMode: Text.WordWrap
        textFormat: Text.RichText
    }

    Button{
        id:pagingButton

        width: parent.width
        height: 60
        visible: isLastPage == false

        anchors{
            top: textPart.bottom
            left: parent.left
            right: parent.right
        }
        onClicked:{
            page = page + 1
            container.contentY = textPart.y - titleBarHeight
        }

        Text{
            text: getPagingIndex()
            color:props.theme.linkColor
            // these lines center that text, but the Text component handles right-to-left
            // so we do not need to center it
            //width: parent.width
            //horizontalAlignment: Text.AlignHCenter
            font {
                pixelSize: 30
                bold: true
            }
            anchors{
                top: pagingButton.top
                left: pagingButton.left
                margins:8
            }
        }
    }

    Button{
        id:linkButton
        width: parent.width
        anchors{
            top: pagingButton.bottom
            left:parent.left
            right:parent.right
        }
        onClicked:loadWebView(container.itemLink,true)

        Text{
            text: props.theme.seeOriginalStory
            color:props.theme.linkColor
            font {
                pixelSize: 30
                bold: true
            }
            anchors{
                top: linkButton.top
                left: linkButton.left
                margins:8
            }
        }
    }

}
