import QtQuick 1.0
import "Util.js" as Util

Rectangle {
    id: container

    property bool readyToRefreshList: false
    property bool readyToRefreshGrid: false
    property string fontName: "Helvetica"
    property int fontSizeTitle:19
    property int fontSizeDescription:17
    property int fontSizeTimestamp:12
    property string itemDescription: ""
    property string itemUrl: ""
    property string itemImageUrl: ""
    property string itemPubmsago:""
    property string filter: ""
    property int spacing: 10
    property RssModel rssModel
    property color listTitleColor : props.theme.linkColor
    property alias  list:listOfItem
    property alias  grid:gridOfItem
    property double pull2refresh_offset: mainWindow.height >mainWindow.width ? 0.9 : 1.2

    signal feedItemSelected(int index)
    signal launchWebView(string advertUri,bool isOrigStory)

    width: screenWidth
    height: screenHeight
    color: "white"

    Component {

        id: listDelegate
        //item of each single cell

        ListItem {

            text: title
            longText: description
            timestamp: pubDate
            width: container.width
            //height: 120
            thumbnailUrl: thumbnail
            currentIndex:index
            fontColorTitle: listTitleColor
            onClicked: {
                //Util.log("ARTICLE GALLERY IS FALLSe")
                // passes list item locals to full item view
                listOfItem.focus = true // Unfocus text field
                appState.currentFeedItemIndex = index
                //console.log("THE CURRENT VIEW IS :" + appState.currentViewName)
                feedItemSelected(index)
                appState.indexCount = listOfItem.count
                appState.isArticleGallery = false
                appState.isMenuAtIndex = false
            }

            function getTitle(index,title){

                if(index == 0){
                    appState.titleFirstItem = title
                }
                return title
            }
        }
    }


    Component {
        id: spotlightDelegate
        //item of each single cell

        SpotlightItem {

            text: title
            longText: description
            timestamp: pubDate
            width: container.width
            thumbnailUrl: thumbnail
            currentIndex:index
            fontColorTitle: listTitleColor
            onClicked: {
                // passes list item locals to full item view
                listOfItem.focus = true // Unfocus text field
                appState.currentFeedItemIndex = index
                feedItemSelected(index)
                appState.indexCount = listOfItem.count
                appState.isArticleGallery = false
                appState.isMenuAtIndex = false

            }
        }
    }

    Component {

        id: galleryDelegate
        //item of each single cell

        GalleryItem {

            width: mainWindow.height > mainWindow.width ? 120 : 122
            height: mainWindow.height > mainWindow.width ? 120 : 122
            thumbnailUrl: thumbnail
            currentIndex:index
            fontColorTitle: listTitleColor

            onClicked: {
                // passes list item locals to full item view
                listOfItem.focus = true // Unfocus text field
                appState.currentFeedItemIndex = index
                feedItemSelected(index)
                appState.isArticleGallery = true
                appState.isMenuAtIndex = false
                appState.indexCount = gridOfItem.count
            }

        }

    }



    Timer {
        id: reloadTimer
        interval: 1000; running: false; repeat: false
        onTriggered: {
            rssModel.reload(true);
            //appState.loading = true
        }
    }





    Component {
        id: listHeader

        Item {
            width: listOfItem.width
            height: listSectionHeader.height

            Image {
                id: headerLogoImg
                anchors.bottom: parent.top
                anchors.left: parent.left
                anchors.bottomMargin: 10
                anchors.leftMargin: 5
                width:60
                height:60
                source:  "gfx/refresharrow.png"
                opacity: {
                    if (listOfItem.contentY < -5 && listOfItem.contentY > -100)
                        if (Math.abs(listOfItem.verticalVelocity) <= 100.0) {
                            readyToRefreshList=false;
                            return 1;
                        }

                    if ((listOfItem.contentY >= 0)&&(readyToRefreshList)) {
                        readyToRefreshList=false;

                        appState.loading = true;
                        reloadTimer.start();
                    }
                    if (-listOfItem.contentY > 10) return 1;
                    else return 0; }
                Behavior on opacity { NumberAnimation { duration: 100  } }
                // binding warning here
                rotation: {
                    var newAngle = - listOfItem.contentY * pull2refresh_offset;
                    if (newAngle > 180) {
                        //Util.log("readytorefreshList: true!");
                        readyToRefreshList = true;
                        return -180;
                    } else {
                        if (readyToRefreshList)
                            return -180;
                        newAngle = newAngle > 180 ? 180 : 0;
                        return -newAngle;
                    }
                }
                Behavior on rotation { NumberAnimation { duration: 150 } }
            }
            Text {
                text:  (readyToRefreshList == false) ? props.theme.pull2Refresh : props.theme.release2Refresh
                wrapMode: Text.WordWrap
                anchors.top: headerLogoImg.top
                anchors.left: headerLogoImg.right
                anchors.bottom: headerLogoImg.bottom
                anchors.right: parent.right
                verticalAlignment: Text.AlignVCenter
                anchors.leftMargin: 35
                font.pixelSize: 24
                font.bold: true
            }
            SectionHeader {

                id: listSectionHeader

                onLaunchWebView: {container.launchWebView(advertClickUrl,false)}
            }
        }
    }


    Component {
        id: gridHeader

        Item {
            width: gridOfItem.width
            height: gridSectionHeader.height
            Image {
                id: headerLogoImg2
                anchors.bottom: parent.top
                anchors.left: parent.left
                anchors.bottomMargin: 10
                anchors.leftMargin: 5
                width:60
                height:60
                source:  "gfx/refresharrow.png"
                opacity: {
                    if (gridOfItem.contentY < -5 && gridOfItem.contentY > -100)
                        if (Math.abs(gridOfItem.verticalVelocity) <= 100.0) {
                            readyToRefreshGrid=false;
                            return 1;
                        }

                    if ((gridOfItem.contentY >= 0)&&(readyToRefreshGrid)) {
                        readyToRefreshGrid=false;

                        appState.loading = true
                        reloadTimer.start();
                    }
                    if (-gridOfItem.contentY > 10) return 1;
                    else return 0;
                }
                Behavior on opacity { NumberAnimation { duration: 100  } }
                rotation: {
                    var newAngle = - gridOfItem.contentY * pull2refresh_offset;
                    if (newAngle > 180) {
                        //Util.log("readytorefreshGrid: true!");
                        readyToRefreshGrid = true;
                        return -180;
                    } else {
                        if (readyToRefreshGrid)
                            return -180;
                        newAngle = newAngle > 180 ? 180 : 0;
                        return -newAngle;
                    }
                }
                Behavior on rotation { NumberAnimation { duration: 150 } }
            }
            Text {
                text:  readyToRefreshGrid == false ? props.theme.pull2Refresh : props.theme.release2Refresh
                wrapMode: Text.WordWrap
                anchors.top: headerLogoImg2.top
                anchors.left: headerLogoImg2.right
                anchors.bottom: headerLogoImg2.bottom
                anchors.right: parent.right
                verticalAlignment: Text.AlignVCenter
                anchors.leftMargin: 35
                font.pixelSize: 24
                font.bold: true
            }
            SectionHeader {

                id: gridSectionHeader

                onLaunchWebView: {container.launchWebView(advertClickUrl,false)}
            }
        }
    }


    ListView {
        id: listOfItem

        cacheBuffer: 3000

        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            bottomMargin: 0
            fill: parent
        }
        header: listHeader
       // footer:footer
        clip: true
        model: rssModel.rssItems
        delegate: appState.templateType == 2 ? spotlightDelegate : listDelegate
        interactive: appState.loading == true? false : true
        visible: !rssModel.isLoading  && appState.templateType != 1
    }


    GridView {
        id: gridOfItem

        cacheBuffer: 500

        anchors {
            top: parent.top
            left: parent.left
            right: parent.right
            bottom: parent.bottom
            bottomMargin: 0
            fill: parent
        }
        header: gridHeader
       // footer:footer
        clip: true
        model: rssModel.rssItems
        delegate: galleryDelegate
        cellHeight: mainWindow.height > mainWindow.width ? 120 : 122
        cellWidth: mainWindow.height > mainWindow.width ? 120 : 122
        interactive: appState.loading == true? false : true
        visible: !rssModel.isLoading && appState.templateType == 1

    }


    Component{
        id: footer
        Footer {
            onOviButtonClicked: container.launchWebView(props.theme.oviUrl,false)
        }
    }

    function activationComplete() {
        // Clear textEntry focus on activation.
        list.focus = true
    }
}
