import QtQuick 1.1
import com.meego 1.0

import "Util.js" as Util

Item {
    id: container

    //colors
    property color fontColorTitle
    property color fontColorTimestamp
    property bool fontBold: false
    property string text: "NOT SET"
    property string longText: "NOT SET"
    property string timestamp:"NOT SET"
    property string thumbnailUrl
    property bool selected: false
    property bool selectable: false
    property string icon: ""
    property double iconOpacity: 1
    property int currentIndex
    property int imageWidth: 120
    property int imageHeight: 120

    // anchors must reference components in the same hierarchy,
    // thus we cannot pass them in from outside this component
    property bool anchorTitleBelowImage: false
    property bool anchorPubDateRightImage: true

    // not a huge fan of exposing all these members
    property alias titlePixelSize: itemTitle.font.pixelSize
    property alias boldFontTitle: itemTitle.font.bold

    signal clicked
    signal pressAndHold(int mouseX, int mouseY)

    LayoutMirroring.enabled: props.theme.rightToLeftOrientation
    LayoutMirroring.childrenInherit: true

    width: mainWindow.width
    height: getHeightForCell()
    clip: true

    onSelectedChanged: selected ?  state = 'selected' : state = ''

    BusyRemoteImage {
        id: iconId
        imageUrl: container.thumbnailUrl
        height: imageHeight
        width: imageWidth
        fillMode: "PreserveAspectFit"
        smooth: true
        opacity: container.iconOpacity
        asynchronous: true
        anchors {
            left:  parent.left
            top: parent.top
            topMargin: 10
            bottomMargin: 15
            leftMargin: 16
        }
    }


    Text {
        id: itemTitle
        property bool anchorTitleToLeft: anchorTitleBelowImage || !hasThumbnailImage()
        wrapMode: Text.Wrap
        textFormat: Text.RichText
        color: "black"
        text: container.text
        verticalAlignment: Text.AlignLeft

        anchors {
            left: anchorTitleToLeft ? container.left : iconId.right
            top: anchorTitleBelowImage ? iconId.bottom : container.top
            right: parent.right
            topMargin: 8
            leftMargin: 16
            rightMargin: 10
        }
        font {
            pixelSize: 28
            bold: true
        }
    }

    Text {
        id: itemDescription
        // only visible in landscape
        visible: shouldShowDesc()
        wrapMode:Text.WordWrap
        textFormat: Text.RichText
        color: "black"
        text: prepareDescText()

        anchors {
            left: itemTitle.anchors.left
            right: container.right
            top:  itemTitle.bottom
            topMargin: 6
            leftMargin: 16
            rightMargin: 10
        }

        font {
            family: "Helvetica"
            pixelSize: 22
            bold: false

        }
    }


    Text {
        id: itemPubmsago
        color: props.theme.timestampFontColor
        text:  container.getTextForTimestamp(container.timestamp)
        anchors {
            left: itemTitle.anchors.left
            leftMargin: itemTitle.anchors.leftMargin
            // no description text in portrait
            top: shouldShowDesc() ? itemDescription.bottom : itemTitle.bottom
            topMargin: 5
            bottomMargin:10
        }
        font {
            pixelSize: 16
            bold: false
        }
    }


    Image{
        id:separator
        source: mainWindow.height > mainWindow.width ? props.theme.images.divider_index_meego : props.theme.images.divider_index_ls_meego
        //smooth: true
        width: container.width
        asynchronous: true
        anchors {
            bottom:container.bottom
            left:container.left
            right: container.right
        }
    }

    function isPortrait() {
        return mainWindow.height > mainWindow.width
    }

    function hasThumbnailImage() {
        return ((thumbnailUrl !== null) &&
                (thumbnailUrl.length !== 0) &&
                !(iconId.status == Image.Null) &&
                !(iconId.status == Image.Error))
    }

    function getFullTextHeight() {

        var textHeight = itemTitle.height +
            itemTitle.anchors.topMargin +
            itemPubmsago.height + itemPubmsago.anchors.topMargin +
            itemPubmsago.anchors.bottomMargin

        // description only in landscape
        if (!(isPortrait() && hasThumbnailImage()) ) {
            textHeight += itemDescription.height +
                    itemDescription.anchors.topMargin
        }
        return textHeight
    }

    function getFullImageHeight() {
        return imageHeight +
                iconId.anchors.topMargin +
                iconId.anchors.bottomMargin
    }

    function getHeightForCell() {
        var heightOfTextElements = getFullTextHeight()
        var heightOfImageContainer = getFullImageHeight()
        if((hasThumbnailImage()) && (heightOfImageContainer > heightOfTextElements)) {
            return heightOfImageContainer
        } else {
            return heightOfTextElements
        }
    }

    function changeOrientation() {
        if(mainWindow.width > mainWindow.height) {
            return props.theme.images.listItemSeparator_ls
        } else {
            return props.theme.images.listItemSeparator
        }
    }

    function shouldShowDesc() {
        if(!isPortrait()){
            if((container.longText !== null) && (container.longText.length > 0)){
                return true
            }
            else{
                return false
            }
        }
        else{
            if(hasThumbnailImage()){
                return false
            }
            else{
                return true
            }
        }
      //  return (!isPortrait() && (container.longText !== null) &&
       //         (container.longText.length > 0))
    }

    function prepareDescText() {

        var TOTAL_CHAR_LIMIT = 130

        if (shouldShowDesc()) {

            // cut long string such that they account for title text length
            var descCharLimit = TOTAL_CHAR_LIMIT - itemTitle.text.length
            var candidateString = container.longText.substring(0, descCharLimit)
            var indexLastSpace = candidateString.lastIndexOf(" ")
            return textNoSpaced(candidateString.substring(0, indexLastSpace).concat(" ..."))

        } else {

            // would rather set this as undefined or null,
            // empty string is goofy and misleading
            return ""
        }
    }

    function textNoSpaced(smallText){
         smallText = smallText.replace(/<br\/>/g," ")
        return smallText
    }


    function getTextForTimestamp(aTimestamp) {

        if ((aTimestamp !== null) && ("NOT SET" !== aTimestamp)){

            var pubDate = new Date(aTimestamp);
            var now = new Date();
            var millisecondsAgo = now.getTime() - pubDate.getTime();
            var timeString = "";

            var minutesDiff = parseInt(millisecondsAgo / 60000);
            var hours = parseInt(minutesDiff / 60);
            var days = parseInt(hours / 24);
            var months = parseInt(days / 30);
            var years = parseInt(days / 365);
            minutesDiff = minutesDiff - (hours * 60);

            if (years > 0) {
                timeString = (years > 1 ? props.theme.yearPlural : props.theme.yearSingle).replace("{number}", years);
            } else if (months > 0) {
                timeString = (months > 1 ? props.theme.monthPlural : props.theme.monthSingle).replace("{number}", months);
            } else if (days > 0) {
                timeString = (days > 1 ? props.theme.dayPlural : props.theme.daySingle).replace("{number}", days);
            } else if (hours > 0) {
                timeString = (hours > 1 ? props.theme.hourPlural : props.theme.hourSingle).replace("{number}", hours);
            } else // do minutes
            {
                if (minutesDiff == 0)
                    timeString = props.theme.justPublished;
                else
                    timeString = (minutesDiff > 1 ? props.theme.minutePlural : props.theme.minuteSingle).replace("{number}", minutesDiff);
            }

            return timeString;
        } else
            return '';
    }

    //allow table cells selection

    MouseArea {
        id: mouseArea
        anchors.fill: parent
        onClicked: {
            //Util.log("the url of the image is:" + appState.loading)
            container.clicked();
        }
        onReleased: selectable && !selected ? selected = true : selected = false
        onPressAndHold: container.pressAndHold(mouseX,mouseY)
    }

    states: [
        State {
            name: 'pressed'; when: mouseArea.pressed
            //PropertyChanges { target: background; source: bgImagePressed; border { left: 35; top: 35; right: 35; bottom: 10 } }
        },
        State {
            name: 'selected'
            // PropertyChanges { target: background; source: bgImageSelected; border { left: 35; top: 35; right: 35; bottom: 10 } }
        },
        State {
            name: 'active';
            //PropertyChanges { target: background; source: bgImageActive; }
        }
    ]
}

