import QtQuick 1.0

import "Util.js" as Util

Item {

    id: rssModel

    signal contentAvailable(bool availability)

    //  onRssItemsChanged: appState.loading = false

    property string feedUrl
    property alias rssItems: justListModel //xmlListModel
    property int template: -1
    property variant channelTitles: []
    property variant audioMediaCont : []
    property variant videoMediaCont : []

    property  string tmpString

    property bool isLoading: false//(justListModel.status == ListModel.Loading) //(xmlListModel.status == XmlListModel.Loading)




    function saveOfflineStorage(tmpId, tmpXml) {
        if (tmpXml == "")
            return;

        var db = openDatabaseSync(props.theme.mippinID, "1.0", "Offline storage database", 1000000);

        db.transaction(
                    function(tx) {
                        tx.executeSql('CREATE TABLE IF NOT EXISTS Channels(id NUMBER, xml TEXT)');
                        tx.executeSql('DELETE FROM Channels WHERE id >= ?', [ channelTitles.length ]);
                        tx.executeSql('DELETE FROM Channels WHERE id = ?', [ tmpId ]);
                        tx.executeSql('INSERT INTO Channels VALUES(?, ?)', [ tmpId, tmpXml ]);

                        tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsInfo(id NUMBER, title TEXT, template NUMBER, ads NUMBER)');
                        tx.executeSql('DELETE FROM ChannelsInfo WHERE id = ?', [ tmpId ]);
                        tx.executeSql('DELETE FROM ChannelsInfo WHERE id >= ?', [ channelTitles.length ]);
                        tx.executeSql('INSERT INTO ChannelsInfo VALUES(?, ?, ?, ?)', [ tmpId, channelTitles[appState.channelIndex], template , appState.adsEnabled]);
                        tx.executeSql('UPDATE ChannelsInfo SET ads = ?', [ appState.adsEnabled ]);

                        tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsMedia(id NUMBER, chid NUMBER, artid NUMBER, mediatype NUMBER, medialink TEXT)');
                        tx.executeSql('DELETE FROM ChannelsMedia WHERE chid = ?', [ tmpId ]);
                        tx.executeSql('DELETE FROM ChannelsMedia WHERE chid >= ?', [ channelTitles.length ]);
                        if (audioMediaCont.length > 0)
                            for(var i=0; i<audioMediaCont.length; i++)
                                for(var j=0; j<audioMediaCont[i].length; j++)
                                    tx.executeSql('INSERT INTO ChannelsMedia VALUES(?, ?, ?, 1, ?)', [ i, tmpId, i, audioMediaCont[i][j] ]);
                        if (videoMediaCont.length > 0)
                            for(var i=0; i<videoMediaCont.length; i++)
                                for(var j=0; j<videoMediaCont[i].length; j++)
                                    tx.executeSql('INSERT INTO ChannelsMedia VALUES(?, ?, ?, 2, ?)', [ i, tmpId, i, videoMediaCont[i][j] ]);


                        //Util.log("------------------------> saving and updating")

                        //put the rest of the channels too, or update (any info that we have)
                        var rs = tx.executeSql('SELECT id, title, template, ads FROM ChannelsInfo WHERE id <> ? ORDER BY id ASC', [ tmpId ] );
                        for (var i=0; i<channelTitles.length;i++)
                            if (i != appState.channelIndex) {
                                if (i <= rs.rows.length)
                                    tx.executeSql('UPDATE ChannelsInfo SET title = ? WHERE id = ?', [ channelTitles[i], i ]);
                                else
                                    tx.executeSql('INSERT INTO ChannelsInfo VALUES(?, ?, 0, ?)', [ i, channelTitles[i], appState.adsEnabled ]);

                            }



                        tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsArticles(entryid NUMBER, chid NUMBER, title TEXT, description TEXT, pubdate TEXT, thumbnail TEXT, imageurl TEXT, originalurl TEXT, mippinurl TEXT, link TEXT, id TEXT, track TEXT, video TEXT)');
                        tx.executeSql('DELETE FROM ChannelsArticles WHERE chid >= ?', [ channelTitles.length ]);
                        tx.executeSql('DELETE FROM ChannelsArticles WHERE chid = ?', [ tmpId ]);


                        for (var i=0; i< justListModel.count; i++) {
                            //Util.log("saving article : " + i + ". " + justListModel.get(i).title );
                            tx.executeSql('INSERT INTO ChannelsArticles VALUES(?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)',
                                          [
                                              i,
                                              tmpId,
                                              justListModel.get(i).title,
                                              justListModel.get(i).description,
                                              justListModel.get(i).pubDate,
                                              justListModel.get(i).thumbnail,
                                              justListModel.get(i).imageUrl,
                                              justListModel.get(i).originalUrl,
                                              justListModel.get(i).mippinUrl,
                                              justListModel.get(i).link,
                                              justListModel.get(i).id,
                                              justListModel.get(i).track,
                                              justListModel.get(i).video
                                          ]);
                        }

                    }
                    )

        clearOldOfflineStorage();
    }


    function clearOldOfflineStorage() {

        var db = openDatabaseSync(props.theme.mippinID, "1.0", "Offline storage database", 1000000);

        var tmpTemplates = new Array();

        db.transaction(
                    function(tx) {
                        var validhashes = new Array();

                        tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsInfo(id NUMBER, title TEXT, template NUMBER, ads NUMBER)');
                        var rs2 = tx.executeSql('SELECT id, title, template, ads FROM ChannelsInfo ORDER BY id ASC' );
                        if (rs2.rows.length == 0) {
                            return;
                        }
                        for (var i=0; i<rs2.rows.length; i++)
                            tmpTemplates.push(rs2.rows.item(i).template);

                        tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsArticles(entryid NUMBER, chid NUMBER, title TEXT, description TEXT, pubdate TEXT, thumbnail TEXT, imageurl TEXT, originalurl TEXT, mippinurl TEXT, link TEXT, id TEXT, track TEXT, video TEXT)');
                        var rsarticles = tx.executeSql('SELECT * FROM ChannelsArticles' );
                        for (var j=0; j<rsarticles.rows.length; j++) {
                            validhashes.push(Qt.md5(rsarticles.rows.item(j).imageurl) + ".png");
                            validhashes.push(Qt.md5(rsarticles.rows.item(j).thumbnail) + ".png");
                            if (tmpTemplates[rsarticles.rows.item(j).chid] == 2)
                                validhashes.push(Qt.md5(rsarticles.rows.item(j).thumbnail.replace("w=80&h=60","w=340&h=220")) + ".png");
                        }

                        imageSaver.clearOldContent(validhashes);

                    });
    }



    function loadOfflineStorage(tmpId) {
        //!!!!
        //Util.log("------------------------> searching for id:" + tmpId)
        var db = openDatabaseSync(props.theme.mippinID, "1.0", "Offline storage database", 1000000);

        var tmpTemplate = 0;


        db.transaction(
                    function(tx) {
                        tx.executeSql('CREATE TABLE IF NOT EXISTS Channels(id NUMBER, xml TEXT)');
                        var rs = tx.executeSql('SELECT id, xml FROM Channels WHERE id = ?', [ tmpId ] );

                        if (rs.rows.length == 0) {
                            //Util.log("------------------------> NO CONTENT IN STORAGE SORRY!")
                            rssModel.contentAvailable(false);
                            return;
                        }
                        else {
                            //Util.log("------------------------> FOUND CONTENT")

                            tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsInfo(id NUMBER, title TEXT, template NUMBER, ads NUMBER)');
                            var rs2 = tx.executeSql('SELECT id, title, template, ads FROM ChannelsInfo ORDER BY id ASC' );

                            if (rs2.rows.length == 0) {
                                //Util.log("------------------------> NO CONTENT IN STORAGE SORRY!")
                                rssModel.contentAvailable(false);
                                return;
                            }

                            try {
                                //Util.log("------------------------> " + rs2.rows.length + " entries in channelsInfo")
                                var chTitles = new Array();
                                for (var i=0; i<rs2.rows.length; i++) {
                                    //Util.log("------------------------> " + " (id: " + rs2.rows.item(i).id + ") " + rs2.rows.item(i).title);
                                    chTitles.push(rs2.rows.item(i).title);
                                }
                                tmpTemplate = rs2.rows.item(tmpId).template;
                                appState.adsEnabled = rs2.rows.item(tmpId).ads;
                                channelTitles = chTitles;

                            }
                            catch (e) {
                                //Util.log("------------------------> oops2!!" + e);
                                tmpTemplate = 0;
                            }

                            template = tmpTemplate



                            //-------------------------------------------------- load media
                            var tmpAudioMediaCont = new Array();
                            var tmpVideoMediaCont = new Array();
                            for (var i=0;i<20;i++) {
                                tmpAudioMediaCont.push(new Array());
                                tmpVideoMediaCont.push(new Array());
                            }
                            tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsMedia(id NUMBER, chid NUMBER, artid NUMBER, mediatype NUMBER, medialink TEXT)');
                            var rsaudio = tx.executeSql('SELECT id, chid, artid, medialink FROM ChannelsMedia WHERE mediatype=1 AND chid=? ORDER BY id ASC', [ tmpId ] );
                            for (var i=0; i<rsaudio.rows.length; i++)
                                tmpAudioMediaCont[rsaudio.rows.item(i).artid].push(rsaudio.rows.item(i).medialink);
                            var rsvideo = tx.executeSql('SELECT id, chid, artid, medialink FROM ChannelsMedia WHERE mediatype=2 AND chid=? ORDER BY id ASC', [ tmpId ]);
                            for (var i=0; i<rsvideo.rows.length; i++)
                                tmpVideoMediaCont[rsvideo.rows.item(i).artid].push(rsvideo.rows.item(i).medialink);
                            audioMediaCont = tmpAudioMediaCont;
                            videoMediaCont = tmpVideoMediaCont;



                            tx.executeSql('CREATE TABLE IF NOT EXISTS ChannelsArticles(entryid NUMBER, chid NUMBER, title TEXT, description TEXT, pubdate TEXT, thumbnail TEXT, imageurl TEXT, originalurl TEXT, mippinurl TEXT, link TEXT, id TEXT, track TEXT, video TEXT)');
                            var rsarticles = tx.executeSql('SELECT * FROM ChannelsArticles WHERE chid=? ORDER BY entryid ASC', [ tmpId ] );
                            for (var j=0; j<rsarticles.rows.length; j++) {
                                //Util.log("loading article : " + j + ". " + rsarticles.rows.item(j).title );
                                justListModel.append({
                                                     'title': rsarticles.rows.item(j).title,
                                                     'description': rsarticles.rows.item(j).description,
                                                     'pubDate': rsarticles.rows.item(j).pubdate,
                                                     'thumbnail': rsarticles.rows.item(j).thumbnail,
                                                     'imageUrl': rsarticles.rows.item(j).imageurl,
                                                     'originalUrl': rsarticles.rows.item(j).originalurl,
                                                     'mippinUrl': rsarticles.rows.item(j).mippinurl,
                                                     'link': rsarticles.rows.item(j).link,
                                                     'id': rsarticles.rows.item(j).id,
                                                     'track': rsarticles.rows.item(j).track,
                                                     'video': rsarticles.rows.item(j).video
                            });
                            }



                            parseDataAndFreeUI("", rs.rows.item(0).xml, false);
                        }
                    }
                    )
    }


    /*
    WorkerScript {
        id: bgthread
        onMessage: {
            rssModel.contentAvailable(true);
            Util.log("------------------------> loading of content done..")
        }
    }
    */



    property bool httpGotResponse : false
    Timer {
        id:httpRequestTimer
        interval: 24000; running: false; repeat: false
        onTriggered: {
            if (!httpGotResponse) {
                Util.xhr.abort();
                props.theme.iAmOnline = false;
                httpGotResponse = true;
            }
        }
    }


    function reload(forceNetReload) {

        httpRequestTimer.stop();
        httpGotResponse = false;



        justListModel.clear();


        appState.loading = true;
        appState.networkError = false
        //console.log("NETWORK ERROR IS SET TO FALSE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")


        if (!forceNetReload) {
            if (mainWindow.channelWasLoaded[appState.channelIndex]){
                //Util.log("************************************************************** channel loaded before, loading offline");
                loadOfflineStorage(appState.channelIndex);
                return;
            }
        }


        //Util.log("************************************************************** first time loading, or FORCED, ONLINE");

        //log the channel loading


        // download rss xml,
        //xhr is globar var in Util.js

        if (Util.xhr != null) {
            Util.xhr.abort();
            Util.xhr = null;
        }

        Util.xhr = new XMLHttpRequest()
        Util.xhr.open("GET", rssModel.feedUrl + "&nocache=" + Qt.formatDateTime(new Date(), "hhmmzzz"), true)

        //var xmlHttpTimeout=setTimeout( function(){ Util.log("timeout");} ,5000);



        Util.xhr.onreadystatechange = function() {

            if (Util.xhr.readyState ==
                    XMLHttpRequest.DONE) {


                httpGotResponse = true;
                httpRequestTimer.stop();

                //appState.networkError = false

                //Util.log("------------------------> response from: " + feedUrl)
                var root = "";

                if(Util.xhr.responseXML == null){
                    props.theme.iAmOnline = false;
                    //Util.log("------------------------> NO NET RESPONSE! TRYING TO LOAD FROM STORAGE" + appState.channelIndex);
                    loadOfflineStorage(appState.channelIndex);
                   // console.log("NETWORK ERROR IS SET TO TRUE!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!")
                    appState.networkError = true
                    return;
                }


                props.theme.iAmOnline = true;

                //Util.log("------------------------> GOT RESPONSE FROM NET, REFRESHING")
                root = Util.xhr.responseXML.documentElement;

                parseDataAndFreeUI(root, Util.xhr.responseText, true);

                var tmp = mainWindow.channelWasLoaded;
                tmp[appState.channelIndex] = true;
                mainWindow.channelWasLoaded = tmp;
            }

            //Util.log("....................................>>>>>> " + Util.xhr.readyState);
        }


        httpRequestTimer.stop();
        httpRequestTimer.start();
        httpGotResponse = false;
        Util.xhr.send()
    }





    function parseDataAndFreeUI(rootXML, rootTxt, onlineContent) {

        //Util.log("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ type is : " + typeof(rootXML));

        if (onlineContent) {
            //Util.log("------------------------> online content!!!")
            try {
                var chTitles = new Array();
                var channel = rootXML.childNodes[1];

                var tempArray = new Array()
                var tempVideoArray = new Array()

                var articlesDict = new Array();

                for (var i = 0; i < channel.childNodes.length; i++ ) {

                    var baby = channel.childNodes[i]
                    // parse out [template and channel titles],
                    if (baby.nodeName == "template") {

                        //console.log("template is " + baby.childNodes[0].nodeValue);
                        template = baby.childNodes[0].nodeValue;

                    } else if (baby.nodeName == "channelTitle") {

                        //console.log("channel is titled " + baby.childNodes[0].nodeValue);
                        chTitles.push(baby.childNodes[0].nodeValue);
                    } else if (baby.nodeName == "ads") {
                        appState.adsEnabled = baby.childNodes[0].nodeValue;
                    }

                    //multimedia support

                    else if(baby.nodeName == "item"){

                        var _title = "";
                        var _description = "";
                        var _pubDate = "";
                        var _thumbnail = "";
                        var _imageUrl = "";
                        var _originalUrl = "";
                        var _mippinUrl = "";
                        var _link = "";
                        var _id = "";
                        var _track = "";
                        var _video = "";

                        var multimediaContainer = new Array()
                        var videoContainer = new Array()

                        for (var a = 0; a < baby.childNodes.length; a++ ) {

                            var tmpNode = baby.childNodes[a];

                            if (tmpNode.nodeName == "#text")
                                continue;

                            //Util.log("--> NODENAME : " + tmpNode.nodeName);
                            //if (tmpNode.nodeValue != null)
                            //    Util.log("-->    VALUE : " + tmpNode.nodeValue);

                            if(tmpNode.nodeName == "title") {

                                _title = tmpNode.childNodes[0].nodeValue;
                                multimediaContainer.push(tmpNode.childNodes[0].nodeValue);
                                videoContainer.push(tmpNode.childNodes[0].nodeValue);

                            } else if(tmpNode.nodeName == "description") {

                               _description = tmpNode.childNodes[0].nodeValue;

                            } else if(tmpNode.nodeName == "pubDate") {

                               _pubDate = tmpNode.childNodes[0].nodeValue;

                            } else if(tmpNode.nodeName == "mippinlink") {

                               _mippinUrl = tmpNode.childNodes[0].nodeValue;

                            } else if(tmpNode.nodeName == "link") {

                               _link = tmpNode.childNodes[0].nodeValue;

                            } else if(tmpNode.nodeName == "id") {

                               _id = tmpNode.childNodes[0].nodeValue;

                            } else if(tmpNode.nodeName == "track") {

                               _track = tmpNode.childNodes[0].nodeValue;

                            } else if(tmpNode.nodeName == "thumbnail") {

                               _thumbnail = tmpNode.attributes["url"].nodeValue;

                            } else if(tmpNode.nodeName == "image") {

                               _imageUrl = tmpNode.attributes["url"].nodeValue;

                            } else if(tmpNode.nodeName == "enclosure") {

                               _video = tmpNode.attributes["url"].nodeValue;
                               videoContainer.push(tmpNode.attributes["url"].nodeValue)

                            } else if(tmpNode.nodeName == "audio") {

                               multimediaContainer.push(tmpNode.attributes["url"].nodeValue)
                            }
                        }

                        tempArray.push(multimediaContainer)
                        tempVideoArray.push(videoContainer)

                        //Util.log("The video array: " + videoContainer)

                        articlesDict.push({ 'title':_title, 'description':_description, 'pubDate':_pubDate, 'thumbnail':_thumbnail, 'imageUrl':_imageUrl, 'originalUrl':_originalUrl, 'mippinUrl':_mippinUrl, 'link':_link, 'id':_id, 'track':_track, 'video':_video});

                    }
                }

                videoMediaCont = tempVideoArray
                audioMediaCont = tempArray;
                channelTitles = chTitles;


                for (var i=0; i<articlesDict.length; i++)
                    justListModel.append(articlesDict[i]);

                saveOfflineStorage(appState.channelIndex, "not needed anymore");
            }
            catch (e) {
                Util.log("------------------------> oups!!" + e)
            }
        }
        else {
            //if loaded from storage these values are already loaded in channelTitles and template
            //Util.log("------------------------> parsing offline ");
        }



        // set xml property of XmlListModel
        //xmlListModel.xml = rootTxt;

        //remember last loaded channel, will be used to download instead of load when clikcing on the same channel
        mainWindow.lastChannelLoaded = appState.channelIndex

        rssModel.contentAvailable(true);

        //Util.log("------------------------> loading of content done..")

    }

    ListModel {
        id:justListModel
    }
}
