import QtQuick 1.1
import "Components.js" as Util

Rectangle {
    id: container

    //property bool showingBackButton: false
    //property int margin: 0
    //property alias text: titleApp.text
    //property string arrowSelection:""
    property ListModel listModel
    property alias barItem: barItem

    signal exitButtonClicked
    signal loadTheChannel(int indexChannel)
    signal navigationGalleryButtonClicked(int index)
    signal shiftTheTable(bool moveDown)

    //javascript functions

    function dropImage(){
        if(appState.channelCount > 1)
            return true
        else
            return false
    }

    height: barItem.height


    Rectangle {
        id: barItem
        height: topLogo.height + channelBar.height
        width: parent.width
        clip: true
        anchors.fill:parent

        Rectangle{
            id: topLogo

            height: mainWindow.height > mainWindow.width ? 60 : 0
            width: parent.width
            color: props.theme.logoImageBgColor

            Image {
                source: props.theme.logoImage
                fillMode: "PreserveAspectFit"
               // width: 360
                height: 60
                LayoutMirroring.enabled: props.theme.rightToLeftOrientation
                anchors {
                    left: topLogo.left
                    top: topLogo.top
                }
            }

            Image {
                source: props.theme.images.gradient_top_meego
                anchors.fill: parent
            }

        }


        Rectangle{
            id:channelBar

            height:60
            width: parent.width
            anchors {
                top: topLogo.bottom
                left: barItem.left
            }
            color: props.theme.appColor
            LayoutMirroring.enabled: props.theme.rightToLeftOrientation
            LayoutMirroring.childrenInherit: true

            Text{
                id:titleApp
                color: props.theme.titleBarTextColor
                text: appState.channelTitle
                width: channelBar.width - dropIcon.width
                anchors {
                    leftMargin: 12
                    rightMargin: 12
                    left: parent.left
                    right: dropIcon.left
                    top: parent.top
                    bottom: parent.bottom
                }
                font.pixelSize: 32
                font.weight: Font.Light
                verticalAlignment: Text.AlignVCenter
            }

            Image {
                id:dropIcon

                source: props.theme.images.icon_dropdown_meego
                anchors {
                    top: parent.top
                    right: parent.right
                    topMargin: (parent.height - dropIcon.height) / 2
                    rightMargin: anchors.topMargin
                }
                visible: appState.multichannelIsVisible ? false : container.dropImage()
            }

            Image {
                source: props.theme.images.gradient_channel_meego
                anchors.fill: parent
            }
        }
    }

    //customization for multichannel

    TitleBarButton {
        id:titleBarItemMultichannel
        visible: setMultichannelVisible()
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.right: parent.right
        height: parent.height


        onClicked:{
            appState.multichannelIsVisible =  !appState.multichannelIsVisible
            container.shiftTheTable(appState.multichannelIsVisible)
        }


        function setMultichannelVisible(){

            if(appState.channelCount <= 1){
                return false
            }
            else{
                if(appState.currentViewName == "articleController"){
                    if(appState.templateType != 1){
                        return true
                    }
                    else{
                        return false
                    }
                }
                else{
                    return true
                }
            }
        }
    }


    //navigational buttons for gallery view

    TitleBarButton{
        id:buttonGalleryPrevious
        visible: appState.isArticleGallery ? true:false
        width: 57
        height: parent.height
        anchors.top: parent.top
        anchors.left: parent.left

        onClicked: {
            if(appState.currentFeedItemIndex > 0) {
                appState.currentFeedItemIndex = appState.currentFeedItemIndex - 1
            }
            else{
                appState.currentFeedItemIndex = appState.indexCount - 1;
            }
            Util.log("Button gallery previous clicked:" + appState.currentFeedItemIndex)
            container.navigationGalleryButtonClicked(appState.currentFeedItemIndex)
        }
    }
    TitleBarButton{
        id:buttonGalleryNext
        visible: appState.isArticleGallery ? true:false
        width: 57
        height: parent.height
        anchors.top: parent.top
        anchors.left: parent.left
        anchors.leftMargin: 57
        onClicked: {
            if(appState.currentFeedItemIndex < appState.indexCount - 1 ) {
                appState.currentFeedItemIndex = appState.currentFeedItemIndex + 1
            }
            else{
                appState.currentFeedItemIndex = 0;
            }
            Util.log("Button gallery next clicked:" + appState.currentFeedItemIndex)
            container.navigationGalleryButtonClicked(appState.currentFeedItemIndex)
        }
    }
}
