import QtQuick 1.1
import com.meego 1.0

// This item provides waiting indicator, should some long -taking operation be in
// progress. Effectively WaitIndicator just shows some graphics, which rolls on the UI.
//
// Usage: Bind whatever model.loading -property etc. with the rotator's show property.
Rectangle {
    id: rotator
    color:"transparent"
    // Defines whether or not the RoTaToR is shown.
    property bool show: false
    // Defines the size of the rolling graphics.

    // Defines how long to wait before showing the wait indicator (in ms).
    property int delay: 700

//color: "black"

    // The width and height are meant to be replaced by the parent, e.g. by
    // using anchors.fill. So remember to modify these when instantiating
    // a ModalNote. This size defines also the dimming & key capture area.
    // If no dimensions are defined, it'll just use S³ default resolution.
    width: parent.width
    height: parent.height

    // Use opacity to hide/show the indicator. Also animate the enterance/exit
    // by using Behaviour on opacity.
    opacity: 1
    Behavior on opacity { PropertyAnimation { duration: 250 } }

    // Use the Fader to dim the background a bit and bind it also to
    // WaitIndicator's "show" -property.


   BusyIndicator{

       anchors.centerIn: parent
       running: rotator.show
       platformStyle: BusyIndicatorStyle { size: "large" }

   }

    // Allow a small delay before displaying the rotator.
    Timer {
        interval: rotator.delay
        running: rotator.show
        repeat: false
        onTriggered: opacity = 1
    }

    onShowChanged: {
        // Handle rotator exit.

        if (show == false) opacity = 0
    }
}
