import QtQuick 1.1
import com.meego 1.0
import "Util.js" as Util
import ImageSaver 1.0
import com.nokia.extras 1.0


PageStackWindow {
    id: appWindow

    initialPage: mainWindow

    function isPortrait() {

        return mainWindow.height > mainWindow.width
    }

    states: [
        State {
            name: "showStatusBar"
            when: screen.currentOrientation == Screen.Portrait || screen.currentOrientation == Screen.PortraitInverted

            PropertyChanges {
                target: appWindow
                showStatusBar:true
            }
        },

        State {
            name: "hideStatusBar"
            when: screen.currentOrientation != Screen.Portrait
            PropertyChanges {
                target: appWindow
                showStatusBar:false
            }
        }

    ]

    // Follow http://doc.qt.nokia.com/4.7/qml-coding-conventions.html
//    Rectangle {
    Page {
        id: mainWindow

        //property alias webView:webView
        property alias webContainer:webContainer
        //property int heightOfWebView: mainWindow.height
        property variant channelWasLoaded : [false,false,false,false]
        property int lastChannelLoaded : 0
        property alias titleBar: titleBar

        anchors.centerIn: parent
        width: parent.width
        height: parent.height

        state: "atStartup"

        WebContainer {

            id: webContainer

            height: mainWindow.height
            width: mainWindow.width
            z:700
            onConnectionNotAvailable: banner.show()
        }

        //will be used to clear old content on exit
        ImageSaver {
            id: imageSaver
        }

        Loader {
            id: props
            property alias theme: props.item
            source: "Properties.qml"
        }


        WaitIndicator {
            id: waitIndicator
            anchors.fill: contentPane
            z: 400
            show: appState.loading

        }


        /*function getTrackUrl(){
        var feed = props.theme.feedUrl
        var search = feed.search("id=") + 3
        var indexLastSpace = feed.length
        var stringSub = feed.substring(search,indexLastSpace)
        var finalAdvertUrl = "http://test.oviappwizard.com/link/nokia-ads.jsp?id=" + stringSub + "&adnum=1"
        return finalAdvertUrl
    }*/


        RssModel {

            id: rssModel
            feedUrl: props.theme.feedUrl

            onIsLoadingChanged: {
                appState.loading = false
            }
            onContentAvailable: {
                if(availability == true){
                    //Util.log("the content is available")
                    appState.loading = false
                    viewNoConnection.visible = false
                }
                else{
                    //Util.log("the content is not available")
                    appState.loading = false
                    viewNoConnection.visible = true
                }

            }
        }


        // Properties.
        AppStateVars {
            id: appState
            templateType: rssModel.template
        }

        //to manage no connection

        Rectangle{
            id:viewNoConnection
            anchors.top:titleBar.bottom
            anchors.left: parent.left
            anchors.bottom: parent.bottom
            anchors.right: parent.right
            color: "white"
            z: 500
            visible: false
            Text{

                anchors.centerIn: parent.Center
                text: props.theme.networkerror
                color: "black"
                font.pixelSize:30
            }
        }

        Timer {
            id: myReloadTimer
            property bool forceOnline:  false
            interval: 1000; running: false; repeat: false
            onTriggered: {
                rssModel.reload(forceOnline);
            }
        }


        // All views have a title bar
        TitleBar {
            id: titleBar
            z: 610
            anchors {
                top: mainWindow.top
                left: mainWindow.left
                right: mainWindow.right
            }
            width: mainWindow.width
            height: isPortrait() ? 120 : 60
            //showingBackButton: appState.showBackButton
            //text: appState.channelTitle
            //arrowSelection: props.theme.images.arrowSelectionUp
            visible: appState.titleBarIsVisible
            listModel: rssModel.rssItems
            onLoadTheChannel: {
                appState.fromLeft = false;
                appState.currentViewName = "feedView";
                multichannel.anchors.topMargin = -  ((appState.channelCount - 1) * 60)
                appState.loading = true
                myReloadTimer.forceOnline = true
                myReloadTimer.start();
            }

            onNavigationGalleryButtonClicked: {
                articleController.list.positionViewAtIndex(index,ListView.Beginning)
            }
            onShiftTheTable: {
                //Util.log("The height of the table is:" + titleBar.height)
                if(moveDown)
                    shiftDownAnimation.start()
                else
                    multichannel.anchors.topMargin = -  ((appState.channelCount - 1) * 60)
            }

            property variant shiftDownAnimation :
                ParallelAnimation {
                NumberAnimation { target: multichannel; property: "anchors.topMargin"; easing.type: Easing.InOutElastic
                    to: 0 ; duration:500 }
            }
        }




        TableMultichannel{
            id:multichannel
            width:mainWindow.width
            height:((appState.channelCount - 1) * 60)
            z:600
            x:0
            y:titleBar.height - ((appState.channelCount - 1) * 60)
            anchors.left: mainWindow.left
            anchors.top: titleBar.bottom
            anchors.topMargin: -  ((appState.channelCount - 1) * 60)

            visible: appState.multichannelIsVisible
            channelTitles: rssModel.channelTitles
            onChannelSelected: {
                props.theme.feedUrl = constructFeedUrl(channelIndex)
                //take the main view at index
                appState.fromLeft = false;
                appState.currentViewName = "feedView";
                anchors.topMargin = -  ((appState.channelCount - 1) * 60)
                //decide whether to fetch content from the net or get it from db
                if (mainWindow.lastChannelLoaded == channelIndex) {
                    appState.loading = true
                    myReloadTimer.forceOnline = true
                    myReloadTimer.start();
                }
                else {
                    appState.loading = true
                    myReloadTimer.forceOnline = false
                    myReloadTimer.start();
                }
            }

            function constructFeedUrl(channelNumber){
                var newUrl = props.theme.feedUrlTemplate + "&c=" + channelNumber;
                return newUrl
            }
        }

        //background for table multichannel

        Item{
            //color:"transparent"
            id:backgroundLayer
            z:590
            width: parent.width
            height: parent.height
            visible: appState.multichannelIsVisible

            Image {


                anchors.fill: parent

                source: props.theme.images.dimmer_meego


                MouseArea{
                    id:buttonExitChannelSelection
                    anchors.fill: parent
                    onClicked:{

                        appState.multichannelIsVisible = false
                        multichannel.anchors.topMargin = -  ((appState.channelCount - 1) * 60)
                    }
                }
            }
        }

        ArticleController{
            id:articleController
            width: parent.width
            height: parent.height
            rssModel: rssModel  //.rssItems
            opacity: 0
            z:180
            onOpenTheWebView: mainWindow.webContainer.openUrl(url,is_orig_story,true)
        }


        // This item will contain the views that we switch between
        Item {
            id: contentPane

            property int  footerHeight: footer.height
            property int  titleBarHeight: mainWindow.height > mainWindow.width ? titleBar.height : 60

            clip: true
            height: mainWindow.height - titleBarHeight - footerHeight

            anchors {
                top: parent.top
                topMargin:titleBarHeight
                left: mainWindow.left
                right: mainWindow.right

                // bottom: mainWindow.bottom
                margins: 0
            }

            // View switcher component, handles the view switching and animation
            ViewSwitcher {
                id: viewSwitcher
                // Rooted in contentPane
                root: contentPane
                // Start from feedListView
                currentView: feedView
            }

            // Feed view showing selected feed's items
            FeedView {
                id: feedView
                anchors {
                    top: parent.top
                    bottom: parent.bottom
                }
                width:  parent.width
                opacity: 1
                rssModel: rssModel
                onLaunchWebView: mainWindow.webContainer.openUrl(advertUri,isOrigStory,true)
                onFeedItemSelected: {
                    //console.log("checkpoint aaaaaaaaaaaAAAAAAAAAAAAA " + appState.currentViewName)
                    appState.fromLeft = false;
                    appState.currentViewName = "articleController";
                    articleController.list.positionViewAtIndex(index,ListView.Beginning)
                }
            }
        }


        Footer {
            id:footer

            clip: true
            height: 72
            z:200
            visible: appState.footerIsVisible

            anchors {
                left: parent.left
                right: parent.right
                top:contentPane.bottom
            }

            onBackButtonClicked: {
                appState.fromLeft = true;
                appState.currentViewName = "feedView";
                feedView.list.positionViewAtIndex(appState.currentFeedItemIndex,ListView.Center)

            }

            onLoadTheChannel: {
                appState.fromLeft = false;

                appState.loading = true
                myReloadTimer.forceOnline = true
                myReloadTimer.start();

                appState.currentViewName = "feedView";
            }

            onOviButtonClicked: mainWindow.webContainer.openUrl(props.theme.oviUrl,false,true)
        }



        InfoBanner{
            id:banner
            //visible: false
            text: props.theme.networkerror
            iconSource: ""
            width: mainWindow.width * 0.75
            timerShowTime: 4000
            anchors.centerIn: mainWindow
            z:2000
        }

        //---------------------- States ----------------------------//

        // Default state is implicit, all other states are defined here
        states: [
            State {
                name: "atStartup"

                PropertyChanges {
                    // Set all state variable changes to appState
                    target: appState
                    currentViewName:"feedView";
                    isMenuAtIndex:"true"
                }

                StateChangeScript {
                    // script: rssModel.reload()
                    script: {
                        screen.allowedOrientations = Screen.All;
                        appState.loading = true
                        myReloadTimer.forceOnline = false
                        myReloadTimer.start();

                    }

                }
            },

            State {
                name: "noConnectivity"
                when: appState.networkError == true && appState.currentViewName == "feedView";

                PropertyChanges {
                    // Set all state variable changes to appState
                    target: appState;
                    //networkError:false;
                }

                StateChangeScript {
                    // script: rssModel.reload()
                    script: {
                        banner.show() ;
                    appState.networkError = false;
                    }
                }
            },


            State {
                name: "waitIndicator"
                when: appState.loading == true;
                PropertyChanges {
                    // Set all state variable changes to appState
                    target: waitIndicator
                    show:appState.loading
                }
                PropertyChanges {
                    // Set all state variable changes to appState
                    target: appState
                    isArticleGallery:false
                }
            },

            State {
                name: "showingFeedView"
                when: appState.currentViewName ===  "feedView";
                PropertyChanges {
                    // Set all state variable changes to appState
                    target: appState
                    showBackButton: false
                    isArticleGallery:false
                }
                PropertyChanges {
                    target: titleBar
                }
                // Animate the view switch with viewSwitcher
                StateChangeScript { script: viewSwitcher.switchView(feedView, appState.fromLeft, 0); }
            },


            State {
                name: "showingFeedItemView"
                when: appState.currentViewName === "articleController" ;
                PropertyChanges {
                    // Set all state variable changes to appState
                    target: appState
                    showBackButton: true
                }
                PropertyChanges {
                    target: titleBar
                }
                // Animate the view switch with viewSwitcher
                StateChangeScript { script: viewSwitcher.switchView(articleController, appState.fromLeft, 0); }
            }


        ]


    }}
